package hexamap;

import java.io.*;

public class Config implements Serializable {
  public int width     = 10;   // Map's width
  public int height    = 13;   // Map's height (heigth mod 2 <> 0)
  public int nbPlayers = 2;    // Number of players
  public int maxNb     = 256;  // Max number for one hexagon
  public int maxDead   = 6;    // Max death by round
  public int maxProd   = 24;   // Max production in base
  public int maxGo     = 16;   // Max transfert
  public int nbBases   = 3;    // Nb bases at beginning
  public int distTgt   = 3;    // Distance from guns and paras target
  public int size      = 24;   // Hexagons size
  public int nbBuild   = 4;    // Number of build for one base
  public int nbFill    = 3;    // Number of fill for water
  public int view      = 4;    // How far can i see (0 : all) ?
  public int time      = 50;   // Milliseconds between 2 refreshs
  public int sep       = 3;    // Max distance between 2 bases at the beginning
  public int minsep    = 4;    // Min distance between ennemy bases at the beginning
  public int computers = 1;    // Number of non human players
  public int speed     = 8;    // Speed of non human players (0 fast, greater = slower)
  public int maxBases  = 0;    // Max number of bases (0 : no max)
  public int quake     = 1;    // Quake (0:off, 1:on)
  public String server = null; // RMI Server (Master)

  public Config() {
  }

  public Config(String[] args) {
    for (int i=0;i<args.length;i++) readArg(args[i]);
  }

  private void readFile(String fileName) {
    try {
        LineNumberReader lnr = new LineNumberReader(new FileReader(fileName));
        String line;
        while ((line=lnr.readLine())!=null) {
            if (line.charAt(0)!='#') readArg(line);
        }
        lnr.close();
    } catch(Exception e) { e.printStackTrace(); }
  }

  private void readArg(String arg) {
    int idx = arg.indexOf("=");
    if (idx<0) return;
    String name = arg.substring(0,idx).trim().toLowerCase();
    String value =arg.substring(idx+1).trim().toLowerCase();
    if (value.equals("default")) return;
    if (name.equals("cfg")) {
       readFile(value);
       return;
    }
    if (name.equals("server")) {
       server = value;
       return;
    }
    int val = 0;
    try { val = Integer.valueOf(value).intValue();
    } catch(Exception e) { e.printStackTrace(); System.err.println(arg); return; }
    if (name.equals("width")) {
       width = val;
    } else if (name.equals("height")) {
       height = val;
    } else if (name.equals("players")) {
       nbPlayers = val;
    } else if (name.equals("number")) {
       maxNb = val;
    } else if (name.equals("dead")) {
       maxDead = val;
    } else if (name.equals("prod")) {
       maxProd = val;
    } else if (name.equals("trans")) {
       maxGo = val;
    } else if (name.equals("bases")) {
       nbBases = val;
    } else if (name.equals("dist")) {
       distTgt = val;
    } else if (name.equals("build")) {
       nbBuild = val;
    } else if (name.equals("fill")) {
       nbFill = val;
    } else if (name.equals("view")) {
       view = val;
    } else if (name.equals("time")) {
       time = val;
    } else if (name.equals("sep")) {
       sep = val;
    } else if (name.equals("minsep")) {
       minsep = val;
    } else if (name.equals("computers")) {
       computers = val;
    } else if (name.equals("speed")) {
       speed = val;
    } else if (name.equals("maxbases")) {
       maxBases = val;
    } else if (name.equals("quake")) {
       quake = val;
    }
  }

  int getMinNb() {
      return maxNb*90/100;
  }

  int getGunPower() {
      return maxNb/10;
  }
}

