package hexamap;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.rmi.*;
import java.rmi.registry.*;
import java.util.*;

public class Game extends Thread {

  private Board board;
  private RemoteMap map;
  private Config cfg;
  private String server;
  private Computer[] computers;
  private boolean stopped = false;

  public Game(Container cont,Config cfg) throws Exception {
    // Init map
    server = cfg.server;
    if (server==null) {
       if (Xagone.bases!=null) Xagone.bases.removeAllElements();
       if (cfg.nbPlayers-cfg.computers>1) {
           try {
               LocateRegistry.createRegistry(1099);
           } catch(Exception e) {
               try {
                   LocateRegistry.getRegistry();
               } catch(Exception ex) {
                   ex.printStackTrace();
                   System.exit(0);
               }
           }
       }
       map = new XaMap(cfg);
       this.cfg = cfg;
       ((XaMap)map).config();
       for (int i=0;i<20;i++) {
           Xagone xg = getRandomXagone();
           xg.addAlt(2);
           Hexagone[] voisins = xg.getAll();
           for (int j=0;j<6;j++) if (voisins[j]!=null) ((Xagone)voisins[j]).addAlt(1);
       }
       for (int i=0;i<6;i++) {
           Xagone xg = getRandomXagone();
           xg.subAlt(1);
           Hexagone[] voisins = xg.getAll();
           for (int j=0;j<6;j++) if (voisins[j]!=null) ((Xagone)voisins[j]).subAlt(1);
       }
       // Init bases
       for (int id = 1; id <= cfg.nbPlayers; id++) {
           for (int i=0; i < cfg.nbBases; i++) {
               Xagone xg = getRandomXagone();
               int j = 0;
               while (xg.getBase()>0||xg.getAlt()<0||((i>0)&&!xg.findID(id,0,cfg.sep,true))||(j<1000&&xg.findID(id,0,cfg.minsep,false))) { xg = getRandomXagone(); j++; }
               xg.buildBase(true);
               xg.setID(id);
               xg.addNb(8,id);
           }
       }
       // Computers
       computers = new Computer[cfg.computers];
       for (int i=0;i<cfg.computers;i++) {
           computers[i] = new Computer((XaMap)map,map.nextID(),cfg);
           // Alliance by 3 for the moment
           if (i%3==2) {
              for (int j=i-2;j<=i;j++) {
                  for (int k=i-2;k<=i;k++) {
                      if (k!=j) computers[j].addFriend(computers[k].getID());
                  }
              }
           }
       }
       if (cfg.nbPlayers-cfg.computers>1) Naming.rebind("JBATTLE_MAP",map);
    } else {
       map = (RemoteMap)Naming.lookup("rmi://"+cfg.server+"/JBATTLE_MAP");
       this.cfg = map.getConfig();
    }
    cont.add(new JScrollPane(board = new Board(map,map.nextID())));
    SwingUtilities.windowForComponent(cont).addKeyListener(board);
  }

  public void keyEvent(KeyEvent e) {
    if (e.getID()==KeyEvent.KEY_TYPED) board.keyTyped(e);
  }

  private Xagone getRandomXagone() throws RemoteException {
    Xagone xg = null;
    while (xg==null) {
        int ib = (int)(Math.random()*cfg.width);
        int jb = (int)(Math.random()*cfg.height);
        xg = (Xagone)((XaMap)map).getHexagone(ib,jb);
    }
    return xg;
  }

  public void run() {
    try {
        while ((!stopped)&&(!map.allConnected())) try { sleep(100); } catch(InterruptedException e) {}
        if (server!=null) {
           while(true) {
             // Display
             board.repaint();
             try { sleep(cfg.time); } catch(InterruptedException e) {}
           }
        }
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        int cpt = cfg.speed;
        while(!stopped) {
          // Dead
          for (int i=0;i<cfg.width;i++) {
              for (int j=0;j<cfg.height;j++) {
                  Xagone x = (Xagone)((XaMap)map).getHexagone(i,j);
                  if (x!=null) {
                     x.count++;
                     if (x.getID()>0) {
                        map.subNb(x.getMapLocation().x,x.getMapLocation().y,(int)(Math.random()*cfg.maxDead));
                     }
                     if (x.count%5==0) x.clearGunPara();
                  }
              }
          }

          // Production
          Vector bases = Xagone.bases;
          for (int i=0;i<bases.size();i++) {
              Xagone x = (Xagone)bases.elementAt(i);
              map.addNb(x.getMapLocation().x,x.getMapLocation().y,(int)(Math.random()*cfg.maxProd),x.getID());
          }

          // Moves
          for (int i=0;i<cfg.width;i++) {
              for (int j=0;j<cfg.height;j++) {
                  Xagone x = (Xagone)((XaMap)map).getHexagone(i,j);
                  if (x!=null&&x.getID()>0) {
                     if (x.goE())  trans(x,(Xagone)x.getE());
                     if (x.goNE()) trans(x,(Xagone)x.getNE());
                     if (x.goSE()) trans(x,(Xagone)x.getSE());
                     if (x.goW())  trans(x,(Xagone)x.getW());
                     if (x.goNW()) trans(x,(Xagone)x.getNW());
                     if (x.goSW()) trans(x,(Xagone)x.getSW());
                  }
              }
          }

          if (cpt==0) {
             for (int i=0;i<cfg.computers;i++) computers[i].playRound();
             cpt = cfg.speed;
          } else cpt--;

          // Display
          board.repaint();
          try { sleep(cfg.time); } catch(InterruptedException e) {}

          // Init earthquake
          if (((XaMap)map).noQuake()) {
             if ((cfg.quake>0)&&(Math.random()<0.001)) ((XaMap)map).quake();
          } else ((XaMap)map).contQuake();
        }
    } catch(RemoteException re) { re.printStackTrace(); }
  }

  public void halt() {
    stopped = true;
  }

  private void trans(Xagone src,Xagone dest) throws RemoteException {
    int nbt = Math.min(src.getNb(),cfg.maxGo);
    if (dest.getID()==src.getID()) nbt=Math.min(nbt,cfg.maxNb-dest.getNb());
    nbt=(int)(((8+(src.getAlt()-dest.getAlt()))/16.0)*nbt);
    if (nbt<=0) return;
    map.addNb(dest.getMapLocation().x,dest.getMapLocation().y,nbt,src.getID());
    map.subNb(src.getMapLocation().x,src.getMapLocation().y,nbt);
  }
}
