package hexamap;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class GameConfig extends JPanel implements ActionListener {

  private static final String[][] options =
    {{"Width","Map's width","WIDTH"},						{"10","20","30"},
     {"Height","Map's height","HEIGHT"},					{"13","25","37"},
     {"Players","Number of players","PLAYERS"},					{"2","3","4","5","6","7","8"},
     {"Max number","Max number for one hexagon","NUMBER"},			{"128","256","512"},
     {"Max death","Max death by round","DEATH"},				{"6","12","24"},
     {"Max production","Max production in base","PROD"},			{"12","24","48"},
     {"Max transfert","Max transfert by round","GO"},				{"8","16","24","32"},
     {"Bases","Number of bases at the beginning","BASES"},			{"3","4","5","6"},
     {"Target distance","Distance from guns and paras target","TGT"},		{"2","3","4","5"},
     {"Hexagon size","Hexagons size","SIZE"},					{"16","24","32"},
     {"Build","Number of build for one base","BUILD"},				{"1","2","3","4","6"},
     {"Fill","Number of fill for water","FILL"},				{"2","3","4"},
     {"View","How far can i see ?","VIEW"},					{"All","3","4","5","6"},
     {"Time","Milliseconds between 2 refreshs","TIME"},				{"25","50","100"},
     {"Sep","Max distance between 2 bases at the beginning","SEP"},		{"1","2","3","4","8"},
     {"Min sep","Min distance between ennemy bases at the beginning","MINSEP"},	{"2","3","4","5","6"},
     {"Computers","Number of non human players","COMPUTERS"},			{"1","2","3","4","5","6","7"},
     {"Speed","Speed of non human players (0 fast, greater = slower)","SPEED"},	{"0","1","2","4","8","16"},
     {"Max bases","Max number of bases","MAXBASES"},				{"None","4","8","16"},
     {"Quake","Enable Earthquake or not","QUAKE"},				{"On","Off"}};

  private static final String[] constLabels = {"All","None","On","Off"};
  private static final int[] constValues = {0,0,1,0};

  private Hashtable combos;
  private Config cfg;

  public GameConfig(Config cfg) {
    super();
    this.cfg = cfg;
    setLayout(new BorderLayout());

    JPanel p = new JPanel();
    p.setLayout(new GridLayout(options.length,1));
    for (int i=0;i<options.length;i+=2) {
        JLabel jl = new JLabel(options[i][0]);
        jl.setToolTipText(options[i][1]);
        p.add(jl);
    }
    add(p,BorderLayout.WEST);

    p = new JPanel();
    p.setLayout(new GridLayout(options.length,1));
    combos = new Hashtable();
    for (int i=0;i<options.length;i+=2) {
        JComboBox jcb = new JComboBox(options[i+1]);
        jcb.setActionCommand(options[i][2]);
        jcb.addActionListener(this);
        jcb.setToolTipText(options[i][1]);
        p.add(jcb);
        combos.put(jcb.getActionCommand(),jcb);
    }
    add(p,BorderLayout.CENTER);

    init("WIDTH",cfg.width);
    init("HEIGHT",cfg.height);
    init("PLAYERS",cfg.nbPlayers);
    init("NUMBER",cfg.maxNb);
    init("DEATH",cfg.maxDead);
    init("PROD",cfg.maxProd);
    init("GO",cfg.maxGo);
    init("BASES",cfg.nbBases);
    init("TGT",cfg.distTgt);
    init("SIZE",cfg.size);
    init("BUILD",cfg.nbBuild);
    init("FILL",cfg.nbFill);
    init("VIEW",cfg.view);
    init("TIME",cfg.time);
    init("SEP",cfg.sep);
    init("MINSEP",cfg.minsep);
    init("COMPUTERS",cfg.computers);
    init("SPEED",cfg.speed);
    init("MAXBASES",cfg.maxBases);
    init("QUAKE",cfg.quake);

    ((JComboBox)combos.get("COMPUTERS")).setEnabled(false);
  }

  public void actionPerformed(ActionEvent e) {
    String ac = e.getActionCommand();
    int v = valueOf(ac);
    if (ac.equals("WIDTH"))		cfg.width = v;
    else if (ac.equals("HEIGHT"))	cfg.height = v;
    else if (ac.equals("PLAYERS")) {
      cfg.nbPlayers = v;
      init("COMPUTERS",cfg.nbPlayers-1);
    } else if (ac.equals("NUMBER"))	cfg.maxNb = v;
    else if (ac.equals("DEATH"))	cfg.maxDead = v;
    else if (ac.equals("PROD"))		cfg.maxProd = v;
    else if (ac.equals("GO"))		cfg.maxGo = v;
    else if (ac.equals("BASES"))	cfg.nbBases = v;
    else if (ac.equals("TGT"))		cfg.distTgt = v;
    else if (ac.equals("SIZE"))		cfg.size = v;
    else if (ac.equals("BUILD"))	cfg.nbBuild = v;
    else if (ac.equals("FILL"))		cfg.nbFill = v;
    else if (ac.equals("VIEW"))		cfg.view = v;
    else if (ac.equals("TIME"))		cfg.time = v;
    else if (ac.equals("SEP"))		cfg.sep = v;
    else if (ac.equals("MINSEP"))	cfg.minsep = v;
    else if (ac.equals("COMPUTERS"))	cfg.computers = v;
    else if (ac.equals("SPEED"))	cfg.speed = v;
    else if (ac.equals("MAXBASES"))	cfg.maxBases = v;
    else if (ac.equals("QUAKE"))	cfg.quake = v;
  }

  private void init(String ctl,int value) {
    JComboBox jcb = (JComboBox)combos.get(ctl);
    String v = String.valueOf(value);
    int idx = getItemIndex(jcb,v);
    if (idx<0) {
       for (int i=0;i<constValues.length&&idx<0;i++) {
           if (constValues[i]==value) idx = getItemIndex(jcb,constLabels[i]);
       }
    }
    if (idx>=0) jcb.setSelectedIndex(idx);
  }

  private int getItemIndex(JComboBox jcb,String item) {
    for (int i=0;i<jcb.getItemCount();i++) {
        if (jcb.getItemAt(i).equals(item)) return i;
    }
    return -1;
  }

  private int valueOf(String ctl) {
    JComboBox jcb = (JComboBox)combos.get(ctl);
    String v = jcb.getSelectedItem().toString();
    try {
        return Integer.valueOf(v).intValue();
    } catch(Exception e) {
        for (int i=0;i<constLabels.length;i++) if (constLabels[i].equals(v)) return constValues[i];
        return 0;
    }
  }

  public Config getConfig() {
    return cfg;
  }
}
