package hexamap;

import java.rmi.*;
import java.util.*;
import java.awt.*;

public class XaMap extends HexaMap implements Remote,RemoteMap {

  private Hashtable hashCurrent;
  private Hashtable hashTarget;
  private int curid;
  private Config cfg;
  private int quaking;

  public XaMap(Config cfg) throws RemoteException {
    super(cfg.width,cfg.height);
    hashCurrent = new Hashtable();
    hashTarget = new Hashtable();
    curid=0;
    quaking=0;
    this.cfg = cfg;
  }

  public void config() {
    for (int i = 0; i < cfg.height; i++)
        for (int j = 0; j < cfg.width-(i%2); j++) ((Xagone)getHexagone(j,i)).setConfig(cfg);
  }

  protected Hexagone createHexagone(int mi,int mj) {
    return new Xagone(this,mi,mj);
  }

  public void clickAt(int x,int y,int id) throws RemoteException {
    Xagone h = (Xagone)hexagoneAt(x,y);
    if (h!=null&&h.getID()==id) h.linkAt(x,y);
  }

  public void moveOn(int x,int y,int id) throws RemoteException {
    Hexagone h = hexagoneAt(x,y);
    if (h!=null) {
       Point p = h.getLocation();
       if (p!=null) {
          hashTarget.remove(String.valueOf(id));
          Hexagone t = hexagoneAt(p.x+((x-p.x)*cfg.distTgt*2),p.y+((y-p.y)*cfg.distTgt*2));
          if (t!=null) hashTarget.put(String.valueOf(id),t);
       }
       setCurrent(h.getMapLocation().x,h.getMapLocation().y,id);
    }
  }

  public void doKey(char key,int id) throws RemoteException {
    Point cur = getCurrent(id);
    if (cur==null) return;
    Xagone h = (Xagone)getHexagone(cur.x,cur.y);
    if (h!=null) h.doKey(key,id);
  }

  public Point getCurrent(int id) throws RemoteException {
    return (Point)hashCurrent.get(String.valueOf(id));
  }

  public Xagone getTarget(int id) {
    return (Xagone)hashTarget.get(String.valueOf(id));
  }

  public void setTarget(Xagone tgt,int id) {
    hashTarget.remove(String.valueOf(id));
    if (tgt!=null) hashTarget.put(String.valueOf(id),tgt);
  }

  public void setCurrent(int i,int j,int id) {
    hashCurrent.remove(String.valueOf(id));
    hashCurrent.put(String.valueOf(id),new Point(i,j));
  }

  public void setGData(int i,int j,Polygon poly,Point location) throws RemoteException {
    getHexagone(i,j).setGData(poly,location);
  }

  public boolean isCurrent(int i,int j,int id) throws RemoteException {
    Point p = (Point)hashCurrent.get(String.valueOf(id));
    return p==null ? false : i==p.x&&j==p.y;
  }

  public Point[] getBases() throws RemoteException {
    Point[] bases = new Point[Xagone.bases.size()];
    for (int i=0;i<bases.length;i++) bases[i] = ((Hexagone)Xagone.bases.elementAt(i)).getMapLocation();
    return bases;
  }

  public void addNb(int i,int j,int n,int id) throws RemoteException {
    ((Xagone)getHexagone(i,j)).addNb(n,id);
  }

  public void subNb(int i,int j,int n) throws RemoteException {
    ((Xagone)getHexagone(i,j)).subNb(n);
  }

  public int nextID() throws RemoteException {
    return ++curid;
  }

  public boolean allConnected() throws RemoteException {
    return curid>=cfg.nbPlayers;
  }

  public Polygon getPolygon(int i,int j) {
    return ((Xagone)getHexagone(i,j)).getPolygon();
  }

  public Point getLocation(int i,int j) {
    return ((Xagone)getHexagone(i,j)).getLocation();
  }

  public boolean goW(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goW();
  }

  public boolean goNW(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goNW();
  }

  public boolean goSW(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goSW();
  }

  public boolean goE(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goE();
  }

  public boolean goNE(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goNE();
  }

  public boolean goSE(int i,int j) {
    return ((Xagone)getHexagone(i,j)).goSE();
  }

  public Color getColor(int i,int j) {
    return ((Xagone)getHexagone(i,j)).getColor();
  }

  public int getID(int i,int j) {
    return ((Xagone)getHexagone(i,j)).getID();
  }

  public int getNb(int i,int j) {
    return ((Xagone)getHexagone(i,j)).getNb();
  }

  public XaStatus[][] copy(int id) throws RemoteException {
    int w = getWidth();
    int h = getHeight();
    XaStatus[][] cpy = new XaStatus[h][]; 
    for (int i = 0; i < h; i++) {
        cpy[i] = new XaStatus[w-i%2];
        for (int j = 0; j < cpy[i].length; j++) cpy[i][j] = ((Xagone)getHexagone(j,i)).copy(id);
    }
    Point p = getCurrent(id);
    if (p!=null) cpy[p.y][p.x].current = true;
    return cpy;
  }

  public Config getConfig() throws RemoteException {
    return cfg;
  }

  public void quake() {
    quaking=24;
  }

  public void contQuake() {
    quaking--;
    if (quaking==0) {
       int r = (int)(Math.random()*cfg.height);
       int c = (int)(Math.random()*(cfg.width-(r%2)));
       Xagone xg = (Xagone)getHexagone(c,r);
       xg.subAlt(4);
       Hexagone[] all = xg.getAll();
       for (int i=0;i<6;i++) if (all[i]!=null) ((Xagone)all[i]).subAlt(2);
    }
  }

  public boolean noQuake() {
    return quaking==0;
  }

  public int getDx() throws RemoteException {
    return noQuake()?0:5-quaking%10;
  }

  public int getDy() throws RemoteException {
    return noQuake()?0:3-quaking%6;
  }
}
