package hexamap;

import java.awt.*;
import java.util.*;

public class Xagone extends Hexagone {

  public static final Color[] cols = new Color[] {null,Color.red,Color.white,Color.blue,Color.yellow,Color.darkGray,Color.magenta,Color.pink,Color.orange};
  public static Vector bases = new Vector();

  public static final double PI = 3.14159;

  private int base = 0;
  private int id = 0;
  private int nb = 0;
  private int alt = 0;
  private boolean lnw,lne,le,lse,lsw,lw;
  private Vector guns,paras;
  private Config cfg;
  public int count;

  public Xagone(XaMap map,int mi,int mj) {
    super(map,mi,mj);
    guns = new Vector();
    paras = new Vector();
    noLink();
  }

  public void setConfig(Config cfg) {
    this.cfg = cfg;
  }

  public void noLink() {
    lnw = lne = le = lse = lsw = lw = false;
  }

  public int getBase() {
    return base;
  }

  public void buildBase(boolean init) {
    if (init) base = cfg.nbBuild;
    else if (base<cfg.nbBuild) base++;
    if ((base==cfg.nbBuild)&&!bases.contains(this)) bases.add(this);
  }

  public void destroyBase() {
    base = 0;
    bases.remove(this);
  }

  public void setID(int id) {
    this.id = id;
  }

  public int getID() {
    return id;
  }

  public int getNb() {
    return nb;
  }

  public void addNb(int n,int id) {
    if (alt<0) return;
    if (getID()!=id) {
       nb-=n;
       if (nb<=0) {
          setID(id);
          nb=-nb;
          noLink();
       }
    } else nb = Math.min(cfg.maxNb,nb+n);
  }

  public void subNb(int n) {
    nb=Math.max(0,nb-=n);
    if ((nb==0)&&(base<cfg.nbBuild)) {
       if (getE()!=null)  if (((Xagone)getE()).goW()) return;
       if (getNE()!=null) if (((Xagone)getNE()).goSW()) return;
       if (getSE()!=null) if (((Xagone)getSE()).goNW()) return;
       if (getW()!=null)  if (((Xagone)getW()).goE()) return;
       if (getNW()!=null) if (((Xagone)getNW()).goSE()) return;
       if (getSW()!=null) if (((Xagone)getSW()).goNE()) return;
       setID(0);
       noLink();
    }
  }

  public int getAlt() {
    return alt;
  }

  public void addAlt(int n) {
    alt=Math.min(15,alt+n);
     if (alt>=0) setColor(new Color(0,255-(alt*16),0));
  }

  public void subAlt(int n) {
    alt=Math.max(-1,alt-n);
    if (alt>=0) setColor(new Color(0,255-(alt*16),0));
    else {
       setColor(Color.green);
       alt = -cfg.nbFill;
       base = 0;
       setID(0);
       noLink();
    }
  }

  public void linkAt(int x,int y) {
    if (getID()==0) return;
    Point p = getLocation();
    double d = Math.sqrt((x-p.x)*(x-p.x)+(y-p.y)*(y-p.y));
    if (y<p.y) {
       double a = Math.acos((x-p.x)/d);
       if (a<PI/6) { // E
          if (getE()!=null) le = !le ;
       } else if (a<PI/2) { // NE
          if (getNE()!=null) lne = !lne ;
       } else if (a<5*PI/6) { // NW
          if (getNW()!=null) lnw = !lnw ;
       } else { // W
          if (getW()!=null) lw = !lw ;
       }
    } else {
       double a = Math.acos((x-p.x)/d);
       if (a<PI/6) { // E
          if (getE()!=null) le = !le ;
       } else if (a<PI/2) { // SE
          if (getSE()!=null) lse = !lse ;
       } else if (a<5*PI/6) { // SW
          if (getSW()!=null) lsw = !lsw ;
       } else { // W
          if (getW()!=null) lw = !lw ;
       }
    }
  }

  public boolean goW()  { return lw; }
  public boolean goNW() { return lnw; }
  public boolean goSW() { return lsw; }
  public boolean goE()  { return le; }
  public boolean goNE() { return lne; }
  public boolean goSE() { return lse; }

  void linkW(boolean b)  { lw = b; }
  void linkNW(boolean b) { lnw = b; }
  void linkSW(boolean b) { lsw = b; }
  void linkE(boolean b)  { le = b; }
  void linkNE(boolean b) { lne = b; }
  void linkSE(boolean b) { lse = b; }

  void link(Xagone x) {
    if (x==getW())       linkW (true);
    else if (x==getNW()) linkNW(true);
    else if (x==getSW()) linkSW(true);
    else if (x==getE())  linkE (true);
    else if (x==getNE()) linkNE(true);
    else if (x==getSE()) linkSE(true);
  }

  boolean isLinkedTo(Xagone x) {
    return (goW()&&x==getW())||(goNW()&&x==getNW())||(goSW()&&x==getSW())||
           (goE()&&x==getE())||(goNE()&&x==getNE())||(goSE()&&x==getSE());
  }

  int linkCount() {
    int nb = getW()!=null&&((Xagone)getW()).goE()?1:0;
    if (getNW()!=null&&((Xagone)getNW()).goSE()) nb++;
    if (getSW()!=null&&((Xagone)getSW()).goNE()) nb++;
    if (getE() !=null&&((Xagone)getE()).goW())   nb++;
    if (getNE()!=null&&((Xagone)getNE()).goSW()) nb++;
    if (getSE()!=null&&((Xagone)getSE()).goNW()) nb++;
    return nb;
  }

  public void doKey(char key,int id) {
    if ((key!='a'&&key!='f')&&(id!=getID())) return;
    int ret = cfg.getMinNb();
    switch(key) {
      case 'a': // Attack
        if (getNW()!=null&&((Xagone)getNW()).getID()==id) { ((Xagone)getNW()).noLink(); ((Xagone)getNW()).linkSE(true); }
        if (getNE()!=null&&((Xagone)getNE()).getID()==id) { ((Xagone)getNE()).noLink(); ((Xagone)getNE()).linkSW(true); }
        if (getW()!=null&&((Xagone)getW()).getID()==id)   { ((Xagone)getW()).noLink();  ((Xagone)getW()).linkE(true); }
        if (getSW()!=null&&((Xagone)getSW()).getID()==id) { ((Xagone)getSW()).noLink(); ((Xagone)getSW()).linkNE(true); }
        if (getSE()!=null&&((Xagone)getSE()).getID()==id) { ((Xagone)getSE()).noLink(); ((Xagone)getSE()).linkNW(true); }
        if (getE()!=null&&((Xagone)getE()).getID()==id)   { ((Xagone)getE()).noLink();  ((Xagone)getE()).linkW(true); }
        break;
      case 'b': // Build base
        if ((getBase()==cfg.nbBuild)||(getNb()<ret)) return;
        buildBase(false);
        subNb(ret);
        break;
      case 'd': // Down
        if (getNb()<ret) return;
        if (getAlt()==0&&getBase()>0) destroyBase();
        else subAlt(1);
        subNb(ret);
        break;
      case 'f': // Fill
        if (getAlt()>=0) return;
        Hexagone[] v = getAll();
        int tot = 0;
        for (int i=0;i<6;i++) if (v[i]!=null&&((Xagone)v[i]).getID()==id) tot+=((Xagone)v[i]).getNb();
        if (tot<ret) return;
        tot = ret;
        while (tot>0) {
            for (int i=0;i<6;i++) {
                if (v[i]!=null&&((Xagone)v[i]).getID()==id&&((Xagone)v[i]).getNb()>0) {
                   ((Xagone)v[i]).subNb(1);
                   tot--;
                }
            }
        }
        addAlt(1);
        break;
      case 'g': // Gun
        Xagone guned = ((XaMap)map).getTarget(id);
        if (guned==null||guned.getID()==id) return;
        guned.addGun(id);
        int ng = Math.min(getNb(),cfg.getGunPower());
        subNb(ng);
        if (guned.getID()>0) guned.subNb(ng*2);
        count = 1;
        break;
      case 'p': // Para
        Xagone target = ((XaMap)map).getTarget(id);
        if (target==null) return;
        target.addPara(id);
        int np = Math.min(getNb(),cfg.getGunPower());
        subNb(np);
        target.addNb(np,id);
        count = 1;
        break;
      case 'r': // Remove links
        noLink();
        break;
      case 'u': // Up
        if (getNb()<ret||getAlt()==15) return;
        addAlt(1);
        subNb(ret);
        break;
      default:
        break;
    }
  }

  public void addGun(int id) {
    Integer i = new Integer(id);
    if (!guns.contains(i)) guns.add(i);
  }

  public void addPara(int id) {
    Integer i = new Integer(id);
    if (!paras.contains(i)) paras.add(i);
  }

  public boolean isUnderAttack() {
    if (guns.size()>0||paras.size()>0) return true;
    Xagone xx = (Xagone)getE();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goW()) return true;
    xx = (Xagone)getNE();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goSW()) return true;
    xx = (Xagone)getSE();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goNW()) return true;
    xx = (Xagone)getW();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goE()) return true;
    xx = (Xagone)getNW();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goSE()) return true;
    xx = (Xagone)getSW();
    if ((xx!=null)&&(xx.getID()!=getID())&&xx.goNE()) return true;
    return false;
  }

  public void clearGunPara() {
    guns.removeAllElements();
    paras.removeAllElements();
  }

  public XaStatus copy(int id) {
    XaStatus xs = new XaStatus();
    xs.base = getBase();
    xs.current = false;
    xs.color = getColor();
    xs.location = getLocation();
    xs.id = getID();
    xs.nb = getNb();
    xs.alt = getAlt();
    xs.poly = getPolygon();
    xs.goLoc[0] = goE()?getE().getLocation():null;
    xs.goLoc[1] = goNE()?getNE().getLocation():null;
    xs.goLoc[2] = goSE()?getSE().getLocation():null;
    xs.goLoc[3] = goW()?getW().getLocation():null;
    xs.goLoc[4] = goNW()?getNW().getLocation():null;
    xs.goLoc[5] = goSW()?getSW().getLocation():null;
    xs.guns = new int[guns.size()];
    for (int i=0;i<xs.guns.length;i++) xs.guns[i] = ((Integer)guns.elementAt(i)).intValue();
    xs.paras = new int[paras.size()];
    for (int i=0;i<xs.paras.length;i++) xs.paras[i] = ((Integer)paras.elementAt(i)).intValue();
    xs.visible = cfg.view==0 ? true : findID(id,0,cfg.view,true);
    return xs;
  }

  public boolean findID(int id,int l,int max,boolean eq) {
    if (++l>max) return false;
    boolean ret = (getID()==0?false:(getID()==id)==eq);
    if (getE()!=null &&!ret) ret = ((Xagone)getE()).findID(id,l,max,eq);
    if (getNE()!=null&&!ret) ret = ((Xagone)getNE()).findID(id,l,max,eq);
    if (getSE()!=null&&!ret) ret = ((Xagone)getSE()).findID(id,l,max,eq);
    if (getW()!=null &&!ret) ret = ((Xagone)getW()).findID(id,l,max,eq);
    if (getNW()!=null&&!ret) ret = ((Xagone)getNW()).findID(id,l,max,eq);
    if (getSW()!=null&&!ret) ret = ((Xagone)getSW()).findID(id,l,max,eq);
    return ret;
  }

  public Xagone getGunner(int id) {
    Vector v = new Vector();
    Vector w = new Vector();
    v.addElement(this);
    int l = cfg.distTgt;
    // Get all hexagone at the good distance
    while (l>0&&v.size()>0) {
        Vector vv = new Vector();
        while (v.size()>0) {
            Xagone x = (Xagone)v.elementAt(0);
            v.removeElementAt(0);
            Hexagone[] all = x.getAll();
            for (int i=0;i<6;i++) {
                if ((all[i]==null) || v.contains(all[i]) || vv.contains(all[i]) || w.contains(all[i])) continue;
                vv.addElement(all[i]);
            }
            w.addElement(x);
        }
        v = vv;
        l--;
    }
    // remove all hexagone with bad id
    Xagone max = null;
    for (l=0;l<v.size();l++) {
        Xagone x = (Xagone)w.elementAt(l);
        if (x.getID()!=id) continue;
        if (max==null||max.getNb()<x.getNb()) max = x;
    }
    return max;
  }
}
